
#pragma once

#include<windows.h>

#if defined(_MSC_VER) && _MSC_VER > 1000
#pragma warning(disable:4311)
#pragma warning(disable:4312)
#endif

class PointerList
{
protected:
	int size;
	void* list;

	CRITICAL_SECTION criticalSection;

public:

	PointerList();

	bool AddPointer(void* pointer);

	void* GetPointer(int id);

	bool SetPointer(int id,void* pointer);

	bool RemovePointer(int id);

	void RemoveAll();

	int GetID(void* pointer);

	int GetSize();

	~PointerList();

};
